/*
 * Written by Sebastian Tilders
 * Licensed under the GPL (General PUblic License).
 * Project Name: InfoOS
 */
#include "../header/typedefs.h"
#include "../header/interrupts.h"
#include "../header/string.h"
#include "../header/pmm.h"
#include "../header/task.h"
#include "../header/display.h"
#define TASK_CURRENT tasks.current_task
#define TASK_NUM_TASKS tasks.num_tasks
#define TASK_STACK tasks.task_status[TASK_NUM_TASKS].stack
#define TASK_USER_STACK tasks.task_status[TASK_NUM_TASKS].user_stack
#define TASK_STATE tasks.task_status[TASK_NUM_TASKS].task_state
struct task* current_task = 0;
struct task* first_task = 0;
uint32_t task_counter = 0;

// Interrupts, Multitasking
/* 
	Einen Task in den Anfangszustand versetzen, sprich alle register = 0 und Stack initalisieren
*/
struct cpu_state* init_ring0_task(uint32_t* stack, uint8_t* entry)
{
	struct cpu_state new_state = {
	.eax = 0,
	.ebx = 0,
	.ecx = 0,
	.edx = 0,
	.esi = 0,
	.edi = 0,
	.ebp = 0,
	.eip = (uint32_t) entry,
	.cs = 0x8,
	.eflags = 0x202,
	};
	struct cpu_state* state = (void*) (stack + 4096 - sizeof(new_state));
	*state = new_state;
	return state;
}
struct cpu_state* init_ring3_task(uint32_t *stack, uint32_t *userstack, uint8_t *entry)
{
  struct cpu_state new_state = {
    .eax = 0,
    .ebx = 0,
    .ecx = 0,
    .edx = 0,
    .esi = 0,
    .edi = 0,
    .ebp = 0,
    .eip = (uint32_t) entry,
    .cs = 0x18 | 0x03,
    .ss = 0x20 | 0x03,
    .esp = (uint32_t) userstack + 4096,
    .eflags = 0x202,
  };
  struct cpu_state* state = (void*) (stack + 4096 - sizeof(new_state));
  *state = new_state;
  return state;
}
// Taskliste  @TODO: überarbeiten (PID etc. hinzufügen)
// ENDE

struct task* create_task(uint8_t* process, uint16_t ring)
{
	 uint32_t* stack = pmm_alloc();
	struct cpu_state* state;
	if(ring != 0)
	{
	uint32_t* user_stack = pmm_alloc();
	  state = init_ring3_task(stack, user_stack,process);
	}
	else
	{
	  state = init_ring0_task(stack, process);
	}
	uint32_t* addr = 0x0;
	struct task* new_task  = 0x0;
	addr = pmm_alloc();
	new_task = (struct task*) addr;
	new_task->task_state = state;
	new_task->next = first_task;
	first_task = new_task;
	task_counter++;
	return new_task;
}
void init_multi(struct multiboot* mbs_info)
{
	if(mbs_info->mbs_mods_count > 0)
	{
		struct mbs_mods* modules = (struct mbs_mods*) mbs_info->mbs_mods_addr;
		uint8_t* load_addr = (uint8_t*) 0x200000;
		size_t length = modules[0].mod_end - modules[0].mod_start;
		memcpy((char*) load_addr,(char*) modules[0].mod_start,length); 
		create_task((uint8_t*) load_addr,0);
	}
}
// Taskliste abarbeiten @TODO: Priorit�ten einbauen.
struct cpu_state* schedule(struct cpu_state* cpu)
{
	if(task_counter < 1)
	{
		return cpu;
	}
	if(current_task == 0)
 	{
	current_task = first_task;
	return current_task->task_state;
	}
	current_task->task_state = cpu;
	if(current_task->next == 0)
	{
		current_task = first_task;
		return current_task->task_state;
	}
	current_task = current_task->next;
	return current_task->task_state;
}
